@@ -24,13 +24,14 @@
  * Corel Corporation or Corel Corporation Limited."
  */
 
+#include <libodfgen/libodfgen.hxx>
+
+#include <deque>
 #include <locale.h>
 #include <math.h>
 #include <string>
 #include <map>
 
-#include <libodfgen/libodfgen.hxx>
-
 #include "FilterInternal.hxx"
 #include "DocumentElement.hxx"
 
@@ -43,10 +44,6 @@
 
 #include "OdfGenerator.hxx"
 
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif
-
 // Workaround for the incapacity of draw to have multiple page
 // sizes in the same document. Once that limitation is lifted,
 // remove this
@@ -58,7 +55,7 @@ class OdgGeneratorPrivate : public OdfGenerator
 {
 public:
 	OdgGeneratorPrivate();
-	~OdgGeneratorPrivate();
+	~OdgGeneratorPrivate() override;
 
 	GraphicStyleManager &getGraphicManager()
 	{
@@ -67,7 +64,7 @@ public:
 
 	void updatePageSpanPropertiesToCreatePage(librevenge::RVNGPropertyList &propList);
 
-	bool writeTargetDocument(OdfDocumentHandler *pHandler, OdfStreamType streamType);
+	bool writeTargetDocument(OdfDocumentHandler *pHandler, OdfStreamType streamType) override;
 	void _writeSettings(OdfDocumentHandler *pHandler);
 	void _writeStyles(OdfDocumentHandler *pHandler);
 	void _writeAutomaticStyles(OdfDocumentHandler *pHandler, OdfStreamType streamType);
@@ -140,7 +137,7 @@ private:
 OdgGeneratorPrivate::OdgGeneratorPrivate() : OdfGenerator(),
 	mStateStack(),
 	mfMaxWidth(0.0), mfMaxHeight(0.0),
-	mpCurrentPageSpan(0), miPageIndex(0),
+	mpCurrentPageSpan(nullptr), miPageIndex(0),
 	mDummyMasterSlideStorage()
 {
 	pushState();
@@ -164,11 +161,25 @@ void OdgGeneratorPrivate::updatePageSpanPropertiesToCreatePage(librevenge::RVNGP
 		mfMaxHeight=height;
 
 	// generate drawing-page style
+	// TODO: This is duplicated in OdpGenerator.cxx. Refactor.
+	if (!pList["draw:fill"]) // it will be removed in a moment again...
+		pList.insert("draw:fill", "none");
 	librevenge::RVNGPropertyList drawingPageStyle;
+	mFillManager.addProperties(pList, drawingPageStyle);
 	librevenge::RVNGPropertyListVector drawingPageVector;
-	drawingPageStyle.insert("draw:fill", "none");
 	drawingPageVector.append(drawingPageStyle);
 	pList.insert("librevenge:drawing-page", drawingPageVector);
+	// remove the drawing page props
+	std::deque<std::string> removedKeys;
+	librevenge::RVNGPropertyList::Iter i(pList);
+	for (i.rewind(); i.next();)
+	{
+		if ((strncmp(i.key(), "draw:", 5) == 0) || (strncmp(i.key(), "presentation:", 13) == 0)
+		        || (strcmp(i.key(), "style:repeat") == 0))
+			removedKeys.push_back(i.key());
+	}
+	for (std::deque<std::string>::const_iterator it = removedKeys.begin(); it != removedKeys.end(); ++it)
+		pList.remove(it->c_str());
 
 	// do not generate footnote separator data
 	pList.insert("librevenge:footnote", librevenge::RVNGPropertyListVector());
@@ -228,6 +239,7 @@ void OdgGeneratorPrivate::_writeAutomaticStyles(OdfDocumentHandler *pHandler, Od
 		mParagraphManager.write(pHandler, Style::Z_StyleAutomatic);
 		mListManager.write(pHandler, Style::Z_StyleAutomatic);
 		mGraphicManager.write(pHandler, Style::Z_StyleAutomatic);
+		mNumberingManager.write(pHandler, Style::Z_StyleAutomatic);
 		mTableManager.write(pHandler, Style::Z_StyleAutomatic, true);
 	}
 	if ((streamType == ODF_FLAT_XML) || (streamType == ODF_CONTENT_XML))
@@ -238,6 +250,7 @@ void OdgGeneratorPrivate::_writeAutomaticStyles(OdfDocumentHandler *pHandler, Od
 		mParagraphManager.write(pHandler, Style::Z_ContentAutomatic);
 		mListManager.write(pHandler, Style::Z_ContentAutomatic);
 		mGraphicManager.write(pHandler, Style::Z_ContentAutomatic);
+		mNumberingManager.write(pHandler, Style::Z_ContentAutomatic);
 		mTableManager.write(pHandler, Style::Z_ContentAutomatic, true);
 	}
 
@@ -249,6 +262,7 @@ void OdgGeneratorPrivate::_writeStyles(OdfDocumentHandler *pHandler)
 	TagOpenElement("office:styles").write(pHandler);
 	mPageSpanManager.writePageStyles(pHandler, Style::Z_Style);
 
+	mFillManager.write(pHandler);
 	mGraphicManager.write(pHandler, Style::Z_Style);
 	mParagraphManager.write(pHandler, Style::Z_Style);
 	mSpanManager.write(pHandler, Style::Z_Style);
@@ -260,6 +274,7 @@ bool OdgGeneratorPrivate::writeTargetDocument(OdfDocumentHandler *pHandler, OdfS
 {
 	if (streamType == ODF_MANIFEST_XML)
 	{
+		pHandler->startDocument();
 		TagOpenElement manifestElement("manifest:manifest");
 		manifestElement.addAttribute("xmlns:manifest", "urn:oasis:names:tc:opendocument:xmlns:manifest:1.0");
 		manifestElement.write(pHandler);
@@ -270,6 +285,7 @@ bool OdgGeneratorPrivate::writeTargetDocument(OdfDocumentHandler *pHandler, OdfS
 		TagCloseElement("manifest:file-entry").write(pHandler);
 		appendFilesInManifest(pHandler);
 		TagCloseElement("manifest:manifest").write(pHandler);
+		pHandler->endDocument();
 		return true;
 	}
 
@@ -283,6 +299,7 @@ bool OdgGeneratorPrivate::writeTargetDocument(OdfDocumentHandler *pHandler, OdfS
 	docContentPropList.addAttribute("xmlns:draw", "urn:oasis:names:tc:opendocument:xmlns:drawing:1.0");
 	docContentPropList.addAttribute("xmlns:table", "urn:oasis:names:tc:opendocument:xmlns:table:1.0");
 	docContentPropList.addAttribute("xmlns:xlink", "http://www.w3.org/1999/xlink");
+	docContentPropList.addAttribute("xmlns:meta", "urn:oasis:names:tc:opendocument:xmlns:meta:1.0");
 	docContentPropList.addAttribute("xmlns:dc", "http://purl.org/dc/elements/1.1/");
 	docContentPropList.addAttribute("xmlns:svg", "urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0");
 	docContentPropList.addAttribute("xmlns:fo", "urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0");
@@ -322,7 +339,7 @@ bool OdgGeneratorPrivate::writeTargetDocument(OdfDocumentHandler *pHandler, OdfS
 	{
 		TagOpenElement("office:body").write(pHandler);
 		TagOpenElement("office:drawing").write(pHandler);
-		sendStorage(&mBodyStorage, pHandler);
+		sendStorage(mpBodyStorage.get(), pHandler);
 		pHandler->endElement("office:drawing");
 		pHandler->endElement("office:body");
 	}
@@ -362,8 +379,9 @@ bool OdgGenerator::getObjectContent(librevenge::RVNGString const &objectName, Od
 	return mpImpl->getObjectContent(objectName, pHandler);
 }
 
-void OdgGenerator::startDocument(const librevenge::RVNGPropertyList &)
+void OdgGenerator::startDocument(const librevenge::RVNGPropertyList &propList)
 {
+	mpImpl->appendBodySettings(propList);
 }
 
 void OdgGenerator::endDocument()
@@ -381,16 +399,16 @@ void OdgGenerator::setDocumentMetaData(const librevenge::RVNGPropertyList &propL
 	mpImpl->setDocumentMetaData(propList);
 }
 
-void OdgGenerator::defineEmbeddedFont(const librevenge::RVNGPropertyList &/*propList*/)
+void OdgGenerator::defineEmbeddedFont(const librevenge::RVNGPropertyList &propList)
 {
-	// TODO: implement me
+	mpImpl->defineEmbeddedFont(propList);
 }
 
-void OdgGenerator::startPage(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::startPage(const librevenge::RVNGPropertyList &propList)
 {
 	librevenge::RVNGPropertyList pList(propList);
 
-	mpImpl->mpCurrentPageSpan=0;
+	mpImpl->mpCurrentPageSpan=nullptr;
 	if (pList["librevenge:master-page-name"])
 	{
 		mpImpl->mpCurrentPageSpan=mpImpl->getPageSpanManager().get(pList["librevenge:master-page-name"]->getStr());
@@ -409,7 +427,7 @@ void OdgGenerator::startPage(const ::librevenge::RVNGPropertyList &propList)
 		pageName.appendEscapedXML(propList["draw:name"]->getStr());
 	else
 		pageName.sprintf("page%i", mpImpl->miPageIndex);
-	TagOpenElement *pDrawPageOpenElement = new TagOpenElement("draw:page");
+	auto pDrawPageOpenElement = std::make_shared<TagOpenElement>("draw:page");
 	pDrawPageOpenElement->addAttribute("draw:name", pageName);
 	pDrawPageOpenElement->addAttribute("draw:style-name", mpImpl->mpCurrentPageSpan->getDrawingName());
 	pDrawPageOpenElement->addAttribute("draw:master-page-name", mpImpl->mpCurrentPageSpan->getMasterName());
@@ -418,10 +436,10 @@ void OdgGenerator::startPage(const ::librevenge::RVNGPropertyList &propList)
 
 void OdgGenerator::endPage()
 {
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:page"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:page"));
 }
 
-void OdgGenerator::startMasterPage(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::startMasterPage(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->inMasterPage())
 	{
@@ -438,7 +456,7 @@ void OdgGenerator::startMasterPage(const ::librevenge::RVNGPropertyList &propLis
 		PageSpan *pageSpan=mpImpl->getPageSpanManager().add(pList, true);
 		if (pageSpan)
 		{
-			libodfgen::DocumentElementVector *pMasterElements = new libodfgen::DocumentElementVector;
+			auto pMasterElements = std::make_shared<libodfgen::DocumentElementVector>();
 			pageSpan->setMasterContent(pMasterElements);
 			mpImpl->pushStorage(pMasterElements);
 		}
@@ -448,7 +466,7 @@ void OdgGenerator::startMasterPage(const ::librevenge::RVNGPropertyList &propLis
 	if (!ok)
 	{
 		ODFGEN_DEBUG_MSG(("OdgGenerator::startMasterPage: creation of the master page has failed\n"));
-		mpImpl->pushStorage(&mpImpl->mDummyMasterSlideStorage);
+		mpImpl->pushStorage(std::shared_ptr<libodfgen::DocumentElementVector>(&mpImpl->mDummyMasterSlideStorage, libodfgen::DummyDeleter()));
 	}
 	mpImpl->pushState();
 }
@@ -466,12 +484,12 @@ void OdgGenerator::endMasterPage()
 	mpImpl->mDummyMasterSlideStorage.clear();
 }
 
-void OdgGenerator::setStyle(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::setStyle(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->defineGraphicStyle(propList);
 }
 
-void OdgGenerator::startLayer(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::startLayer(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->inMasterPage())
 	{
@@ -485,7 +503,7 @@ void OdgGenerator::startLayer(const ::librevenge::RVNGPropertyList &propList)
 	else
 	{
 		mpImpl->getState().mbInFalseLayerGroup=true;
-		mpImpl->getCurrentStorage()->push_back(new TagOpenElement("draw:g"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("draw:g"));
 	}
 }
 
@@ -495,38 +513,38 @@ void OdgGenerator::endLayer()
 		return;
 
 	if (mpImpl->getState().mbInFalseLayerGroup)
-		mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:g"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:g"));
 	else
 		mpImpl->closeLayer();
 	mpImpl->popState();
 }
 
-void OdgGenerator::openGroup(const ::librevenge::RVNGPropertyList & /* propList */)
+void OdgGenerator::openGroup(const librevenge::RVNGPropertyList & /* propList */)
 {
-	mpImpl->getCurrentStorage()->push_back(new TagOpenElement("draw:g"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("draw:g"));
 }
 
 void OdgGenerator::closeGroup()
 {
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:g"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:g"));
 }
 
-void OdgGenerator::drawRectangle(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::drawRectangle(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawRectangle(propList);
 }
 
-void OdgGenerator::drawEllipse(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::drawEllipse(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawEllipse(propList);
 }
 
-void OdgGenerator::drawPolyline(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::drawPolyline(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawPolySomething(propList, false);
 }
 
-void OdgGenerator::drawPolygon(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::drawPolygon(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawPolySomething(propList, true);
 }
@@ -536,12 +554,15 @@ void OdgGenerator::drawPath(const librevenge::RVNGPropertyList &propList)
 	mpImpl->drawPath(propList);
 }
 
-void OdgGenerator::drawGraphicObject(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::drawGraphicObject(const librevenge::RVNGPropertyList &propList)
 {
-	if (!propList["librevenge:mime-type"] || propList["librevenge:mime-type"]->getStr().len() <= 0)
-		return;
-	if (!propList["office:binary-data"])
-		return;
+	if (!propList["librevenge:xlink"] || propList["librevenge:xlink"]->getStr().empty())
+	{
+		if (!propList["librevenge:mime-type"] || propList["librevenge:mime-type"]->getStr().empty())
+			return;
+		if (!propList["office:binary-data"])
+			return;
+	}
 	if (!propList["svg:x"] || !propList["svg:y"] || !propList["svg:width"] || !propList["svg:height"])
 		return;
 
@@ -584,7 +605,7 @@ void OdgGenerator::drawGraphicObject(const ::librevenge::RVNGPropertyList &propL
 	}
 
 	double angle(propList["librevenge:rotate"] ? - M_PI * propList["librevenge:rotate"]->getDouble() / 180.0 : 0.0);
-	if (angle != 0.0)
+	if (angle < 0 || angle > 0)
 	{
 		double deltax((width*cos(angle)+height*sin(angle)-width)/2.0);
 		double deltay((-width*sin(angle)+height*cos(angle)-height)/2.0);
@@ -599,7 +620,7 @@ void OdgGenerator::drawGraphicObject(const ::librevenge::RVNGPropertyList &propL
 	framePropList.insert("svg:height", height);
 	framePropList.insert("svg:width", width);
 
-	TagOpenElement *pDrawFrameElement = new TagOpenElement("draw:frame");
+	auto pDrawFrameElement = std::make_shared<TagOpenElement>("draw:frame");
 
 	librevenge::RVNGPropertyList finalStyle;
 	mpImpl->getGraphicManager().addGraphicProperties(style, finalStyle);
@@ -611,7 +632,7 @@ void OdgGenerator::drawGraphicObject(const ::librevenge::RVNGPropertyList &propL
 	pDrawFrameElement->addAttribute("svg:height", framePropList["svg:height"]->getStr());
 	pDrawFrameElement->addAttribute("svg:width", framePropList["svg:width"]->getStr());
 
-	if (angle != 0.0)
+	if (angle < 0 || angle > 0)
 	{
 		framePropList.insert("librevenge:rotate", angle, librevenge::RVNG_GENERIC);
 		librevenge::RVNGString sValue;
@@ -626,14 +647,18 @@ void OdgGenerator::drawGraphicObject(const ::librevenge::RVNGPropertyList &propL
 		pDrawFrameElement->addAttribute("svg:x", framePropList["svg:x"]->getStr());
 		pDrawFrameElement->addAttribute("svg:y", framePropList["svg:y"]->getStr());
 	}
+
+	if (propList["draw:display"])
+		pDrawFrameElement->addAttribute("draw:display", propList["draw:display"]->getStr());
+
 	mpImpl->getCurrentStorage()->push_back(pDrawFrameElement);
 
 	mpImpl->insertBinaryObject(propList);
 
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:frame"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:frame"));
 }
 
-void OdgGenerator::drawConnector(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::drawConnector(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawConnector(propList);
 }
@@ -665,7 +690,7 @@ void OdgGenerator::startTextObject(const librevenge::RVNGPropertyList &propList)
 	librevenge::RVNGString sValue=mpImpl->getGraphicManager().findOrAdd
 	                              (graphicStyle, mpImpl->useStyleAutomaticZone() ? Style::Z_StyleAutomatic : Style::Z_ContentAutomatic);
 
-	TagOpenElement *pDrawFrameOpenElement = new TagOpenElement("draw:frame");
+	auto pDrawFrameOpenElement = std::make_shared<TagOpenElement>("draw:frame");
 	pDrawFrameOpenElement->addAttribute("draw:style-name", sValue);
 	pDrawFrameOpenElement->addAttribute("draw:layer", mpImpl->getLayerName(propList));
 
@@ -686,10 +711,10 @@ void OdgGenerator::startTextObject(const librevenge::RVNGPropertyList &propList)
 		"fo:min-width", "fo:min-height", "fo:max-width", "fo:max-height", "fo:padding-top", "fo:padding-bottom",
 		"fo:padding-left", "fo:padding-right", "draw:textarea-vertical-align", "draw:fill", "draw:fill-color"
 	};
-	for (unsigned i=0; i<ODFGEN_N_ELEMENTS(attrib); ++i)
+	for (auto &i : attrib)
 	{
-		if (propList[attrib[i]])
-			pDrawFrameOpenElement->addAttribute(attrib[i], propList[attrib[i]]->getStr());
+		if (propList[i])
+			pDrawFrameOpenElement->addAttribute(i, propList[i]->getStr());
 	}
 
 	double x = 0.0;
@@ -699,7 +724,7 @@ void OdgGenerator::startTextObject(const librevenge::RVNGPropertyList &propList)
 	if (propList["svg:y"])
 		getInchValue(*propList["svg:y"],y);
 	double angle(propList["librevenge:rotate"] ? - M_PI * propList["librevenge:rotate"]->getDouble() / 180.0 : 0.0);
-	if (angle != 0.0)
+	if (angle < 0 || angle > 0)
 	{
 		// compute position: make sure that the center position remains invariant
 		double width = 0.0;
@@ -723,11 +748,11 @@ void OdgGenerator::startTextObject(const librevenge::RVNGPropertyList &propList)
 		x -= deltax;
 		y -= deltay;
 	}
-	shared_ptr<librevenge::RVNGProperty> svg_x(librevenge::RVNGPropertyFactory::newInchProp(x));
-	shared_ptr<librevenge::RVNGProperty> svg_y(librevenge::RVNGPropertyFactory::newInchProp(y));
-	if (angle != 0.0)
+	std::shared_ptr<librevenge::RVNGProperty> svg_x(librevenge::RVNGPropertyFactory::newInchProp(x));
+	std::shared_ptr<librevenge::RVNGProperty> svg_y(librevenge::RVNGPropertyFactory::newInchProp(y));
+	if (angle < 0 || angle > 0)
 	{
-		shared_ptr<librevenge::RVNGProperty> librevenge_rotate(librevenge::RVNGPropertyFactory::newDoubleProp(angle));
+		std::shared_ptr<librevenge::RVNGProperty> librevenge_rotate(librevenge::RVNGPropertyFactory::newDoubleProp(angle));
 		sValue.sprintf("rotate (%s) translate(%s, %s)",
 		               librevenge_rotate->getStr().cstr(),
 		               svg_x->getStr().cstr(),
@@ -741,8 +766,12 @@ void OdgGenerator::startTextObject(const librevenge::RVNGPropertyList &propList)
 		if (propList["svg:y"])
 			pDrawFrameOpenElement->addAttribute("svg:y", svg_y->getStr());
 	}
+
+	if (propList["draw:display"])
+		pDrawFrameOpenElement->addAttribute("draw:display", propList["draw:display"]->getStr());
+
 	mpImpl->getCurrentStorage()->push_back(pDrawFrameOpenElement);
-	mpImpl->getCurrentStorage()->push_back(new TagOpenElement("draw:text-box"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("draw:text-box"));
 
 	// push the different states
 	mpImpl->pushState();
@@ -764,14 +793,14 @@ void OdgGenerator::endTextObject()
 	mpImpl->popListState();
 	mpImpl->popState();
 
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:text-box"));
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:frame"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:text-box"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:frame"));
 }
 
-void OdgGenerator::startTableObject(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::startTableObject(const librevenge::RVNGPropertyList &propList)
 {
 	// table must be inside a frame
-	TagOpenElement *pFrameOpenElement = new TagOpenElement("draw:frame");
+	auto pFrameOpenElement = std::make_shared<TagOpenElement>("draw:frame");
 
 	pFrameOpenElement->addAttribute("draw:style-name", "standard");
 	pFrameOpenElement->addAttribute("draw:layer", mpImpl->getLayerName(propList));
@@ -798,10 +827,10 @@ void OdgGenerator::endTableObject()
 	mpImpl->popListState();
 
 	mpImpl->closeTable();
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:frame"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:frame"));
 }
 
-void OdgGenerator::openTableRow(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::openTableRow(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->openTableRow(propList);
 }
@@ -811,7 +840,7 @@ void OdgGenerator::closeTableRow()
 	mpImpl->closeTableRow();
 }
 
-void OdgGenerator::openTableCell(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::openTableCell(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->getState().mbInTableCell)
 	{
@@ -841,7 +870,7 @@ void OdgGenerator::closeTableCell()
 	mpImpl->getState().mbInTableCell = false;
 }
 
-void OdgGenerator::insertCoveredTableCell(const ::librevenge::RVNGPropertyList &propList)
+void OdgGenerator::insertCoveredTableCell(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->insertCoveredTableCell(propList);
 }
