@@ -29,8 +29,10 @@
 #include <locale.h>
 #include <math.h>
 
+#include <deque>
 #include <string>
 #include <map>
+#include <memory>
 #include <stack>
 
 #include "DocumentElement.hxx"
@@ -45,10 +47,6 @@
 
 #include "OdfGenerator.hxx"
 
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif
-
 // Workaround for the incapacity of impress to have multiple page
 // sizes in the same document. Once that limitation is lifted,
 // remove this
@@ -87,9 +85,9 @@ class OdpGeneratorPrivate : public OdfGenerator
 {
 public:
 	OdpGeneratorPrivate();
-	~OdpGeneratorPrivate();
+	~OdpGeneratorPrivate() override;
 
-	void openTextBoxFrame(const ::librevenge::RVNGPropertyList &propList);
+	void openTextBoxFrame(const librevenge::RVNGPropertyList &propList);
 	void closeTextBoxFrame();
 
 	GraphicStyleManager &getGraphicManager()
@@ -102,7 +100,7 @@ public:
 	void writeNotesStyles(OdfDocumentHandler *pHandler);
 
 	//! returns the document type
-	bool writeTargetDocument(OdfDocumentHandler *pHandler, OdfStreamType streamType);
+	bool writeTargetDocument(OdfDocumentHandler *pHandler, OdfStreamType streamType) override;
 	void _writeSettings(OdfDocumentHandler *pHandler);
 	void _writeStyles(OdfDocumentHandler *pHandler);
 	void _writeAutomaticStyles(OdfDocumentHandler *pHandler, OdfStreamType streamType);
@@ -129,7 +127,7 @@ private:
 
 OdpGeneratorPrivate::OdpGeneratorPrivate() :
 	mfMaxWidth(0.0), mfMaxHeight(0.0),
-	mpCurrentPageSpan(0), miPageIndex(0),
+	mpCurrentPageSpan(nullptr), miPageIndex(0),
 	mState(),
 	mDummyMasterSlideStorage()
 {
@@ -153,17 +151,39 @@ void OdpGeneratorPrivate::updatePageSpanPropertiesToCreatePage(librevenge::RVNGP
 		mfMaxHeight=height;
 
 	// generate drawing-page style
+	// TODO: This is duplicated in OdpGenerator.cxx. Refactor.
+	if (!pList["draw:fill"]) // it will be removed in a moment again...
+		pList.insert("draw:fill", "none");
 	librevenge::RVNGPropertyList drawingPageStyle;
+	mFillManager.addProperties(pList, drawingPageStyle);
+	if (pList["presentation:duration"])
+		drawingPageStyle.insert("presentation:duration", pList["presentation:duration"]->getStr());
+	else if (pList["presentation:transition-speed"])
+		drawingPageStyle.insert("presentation:transition-speed", pList["presentation:transition-speed"]->getStr());
+	if (pList["smil:type"])
+		drawingPageStyle.insert("smil:type", pList["smil:type"]->getStr());
+	if (pList["smil:subtype"])
+		drawingPageStyle.insert("smil:subtype", pList["smil:subtype"]->getStr());
 	librevenge::RVNGPropertyListVector drawingPageVector;
-	drawingPageStyle.insert("draw:fill", "none");
 	drawingPageVector.append(drawingPageStyle);
 	pList.insert("librevenge:drawing-page", drawingPageVector);
+	// remove the drawing page props
+	std::deque<std::string> removedKeys;
+	librevenge::RVNGPropertyList::Iter i(pList);
+	for (i.rewind(); i.next();)
+	{
+		if ((strncmp(i.key(), "draw:", 5) == 0) || (strncmp(i.key(), "smil:", 5) == 0) ||
+		        (strncmp(i.key(), "presentation:", 13) == 0) || (strcmp(i.key(), "style:repeat") == 0))
+			removedKeys.push_back(i.key());
+	}
+	for (std::deque<std::string>::const_iterator it = removedKeys.begin(); it != removedKeys.end(); ++it)
+		pList.remove(it->c_str());
 
 	// do not generate footnote separator data
 	pList.insert("librevenge:footnote", librevenge::RVNGPropertyListVector());
 }
 
-void OdpGeneratorPrivate::openTextBoxFrame(const ::librevenge::RVNGPropertyList &propList)
+void OdpGeneratorPrivate::openTextBoxFrame(const librevenge::RVNGPropertyList &propList)
 {
 	librevenge::RVNGPropertyList tmpList(propList), graphicStyle;
 	if (!propList["draw:stroke"])
@@ -175,7 +195,7 @@ void OdpGeneratorPrivate::openTextBoxFrame(const ::librevenge::RVNGPropertyList
 	librevenge::RVNGString sValue=mGraphicManager.findOrAdd
 	                              (graphicStyle, useStyleAutomaticZone() ? Style::Z_StyleAutomatic : Style::Z_ContentAutomatic);
 
-	TagOpenElement *pDrawFrameOpenElement = new TagOpenElement("draw:frame");
+	auto pDrawFrameOpenElement = std::make_shared<TagOpenElement>("draw:frame");
 	pDrawFrameOpenElement->addAttribute("draw:style-name", sValue);
 	pDrawFrameOpenElement->addAttribute("draw:layer", "layout");
 
@@ -196,10 +216,10 @@ void OdpGeneratorPrivate::openTextBoxFrame(const ::librevenge::RVNGPropertyList
 		"fo:min-width", "fo:min-height", "fo:max-width", "fo:max-height", "fo:padding-top", "fo:padding-bottom",
 		"fo:padding-left", "fo:padding-right", "draw:textarea-vertical-align", "draw:fill", "draw:fill-color"
 	};
-	for (unsigned i=0; i<ODFGEN_N_ELEMENTS(attrib); ++i)
+	for (auto &i : attrib)
 	{
-		if (propList[attrib[i]])
-			pDrawFrameOpenElement->addAttribute(attrib[i], propList[attrib[i]]->getStr());
+		if (propList[i])
+			pDrawFrameOpenElement->addAttribute(i, propList[i]->getStr());
 	}
 
 	double x = 0.0;
@@ -209,7 +229,7 @@ void OdpGeneratorPrivate::openTextBoxFrame(const ::librevenge::RVNGPropertyList
 	if (propList["svg:y"])
 		libodfgen::getInchValue(*propList["svg:y"], y);
 	double angle(propList["librevenge:rotate"] ? - M_PI * propList["librevenge:rotate"]->getDouble() / 180.0 : 0.0);
-	if (angle != 0.0)
+	if (angle < 0 || angle > 0)
 	{
 		// compute position: make sure that the center position remains invariant
 		double width = 0.0;
@@ -233,11 +253,11 @@ void OdpGeneratorPrivate::openTextBoxFrame(const ::librevenge::RVNGPropertyList
 		x -= deltax;
 		y -= deltay;
 	}
-	shared_ptr<librevenge::RVNGProperty> svg_x(librevenge::RVNGPropertyFactory::newInchProp(x));
-	shared_ptr<librevenge::RVNGProperty> svg_y(librevenge::RVNGPropertyFactory::newInchProp(y));
-	if (angle != 0.0)
+	std::shared_ptr<librevenge::RVNGProperty> svg_x(librevenge::RVNGPropertyFactory::newInchProp(x));
+	std::shared_ptr<librevenge::RVNGProperty> svg_y(librevenge::RVNGPropertyFactory::newInchProp(y));
+	if (angle < 0 || angle > 0)
 	{
-		shared_ptr<librevenge::RVNGProperty> librevenge_rotate(librevenge::RVNGPropertyFactory::newDoubleProp(angle));
+		std::shared_ptr<librevenge::RVNGProperty> librevenge_rotate(librevenge::RVNGPropertyFactory::newDoubleProp(angle));
 		sValue.sprintf("rotate (%s) translate(%s, %s)",
 		               librevenge_rotate->getStr().cstr(),
 		               svg_x->getStr().cstr(),
@@ -256,7 +276,7 @@ void OdpGeneratorPrivate::openTextBoxFrame(const ::librevenge::RVNGPropertyList
 
 void OdpGeneratorPrivate::closeTextBoxFrame()
 {
-	getCurrentStorage()->push_back(new TagCloseElement("draw:frame"));
+	getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:frame"));
 }
 
 void OdpGeneratorPrivate::writeNotesStyles(OdfDocumentHandler *pHandler)
@@ -377,6 +397,7 @@ void OdpGeneratorPrivate::_writeAutomaticStyles(OdfDocumentHandler *pHandler, Od
 		mParagraphManager.write(pHandler, Style::Z_StyleAutomatic);
 		mListManager.write(pHandler, Style::Z_StyleAutomatic);
 		mGraphicManager.write(pHandler, Style::Z_StyleAutomatic);
+		mNumberingManager.write(pHandler, Style::Z_StyleAutomatic);
 		mTableManager.write(pHandler, Style::Z_StyleAutomatic, true);
 	}
 	if ((streamType == ODF_FLAT_XML) || (streamType == ODF_CONTENT_XML))
@@ -387,6 +408,7 @@ void OdpGeneratorPrivate::_writeAutomaticStyles(OdfDocumentHandler *pHandler, Od
 		mParagraphManager.write(pHandler, Style::Z_ContentAutomatic);
 		mListManager.write(pHandler, Style::Z_ContentAutomatic);
 		mGraphicManager.write(pHandler, Style::Z_ContentAutomatic);
+		mNumberingManager.write(pHandler, Style::Z_ContentAutomatic);
 		mTableManager.write(pHandler, Style::Z_ContentAutomatic, true);
 	}
 
@@ -404,6 +426,7 @@ void OdpGeneratorPrivate::_writeStyles(OdfDocumentHandler *pHandler)
 	mParagraphManager.write(pHandler, Style::Z_Style);
 	mSpanManager.write(pHandler, Style::Z_Style);
 	mListManager.write(pHandler, Style::Z_Style);
+	mFillManager.write(pHandler);
 	mGraphicManager.write(pHandler, Style::Z_Style);
 	pHandler->endElement("office:styles");
 }
@@ -444,9 +467,13 @@ bool OdpGeneratorPrivate::writeTargetDocument(OdfDocumentHandler *pHandler, OdfS
 	docContentPropList.addAttribute("xmlns:fo", "urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0");
 	docContentPropList.addAttribute("xmlns:config", "urn:oasis:names:tc:opendocument:xmlns:config:1.0");
 	docContentPropList.addAttribute("xmlns:xlink", "http://www.w3.org/1999/xlink");
+	docContentPropList.addAttribute("xmlns:meta", "urn:oasis:names:tc:opendocument:xmlns:meta:1.0");
+	docContentPropList.addAttribute("xmlns:number", "urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0");
 	// WARNING: this is not ODF!
 	docContentPropList.addAttribute("xmlns:ooo", "http://openoffice.org/2004/office");
 	docContentPropList.addAttribute("xmlns:officeooo", "http://openoffice.org/2009/office");
+	docContentPropList.addAttribute("xmlns:anim","urn:oasis:names:tc:opendocument:xmlns:animation:1.0");
+	docContentPropList.addAttribute("xmlns:smil","urn:oasis:names:tc:opendocument:xmlns:smil-compatible:1.0");
 	docContentPropList.addAttribute("office:version", "1.0", true);
 	if (streamType == ODF_FLAT_XML)
 		docContentPropList.addAttribute("office:mimetype", "application/vnd.oasis.opendocument.presentation");
@@ -483,7 +510,7 @@ bool OdpGeneratorPrivate::writeTargetDocument(OdfDocumentHandler *pHandler, OdfS
 	{
 		TagOpenElement("office:body").write(pHandler);
 		TagOpenElement("office:presentation").write(pHandler);
-		sendStorage(&mBodyStorage, pHandler);
+		sendStorage(mpBodyStorage.get(), pHandler);
 		pHandler->endElement("office:presentation");
 		pHandler->endElement("office:body");
 	}
@@ -523,8 +550,9 @@ bool OdpGenerator::getObjectContent(librevenge::RVNGString const &objectName, Od
 	return mpImpl->getObjectContent(objectName, pHandler);
 }
 
-void OdpGenerator::startDocument(const ::librevenge::RVNGPropertyList &/*propList*/)
+void OdpGenerator::startDocument(const librevenge::RVNGPropertyList &propList)
 {
+	mpImpl->appendBodySettings(propList);
 }
 
 void OdpGenerator::endDocument()
@@ -537,28 +565,21 @@ void OdpGenerator::endDocument()
 	mpImpl->writeTargetDocuments();
 }
 
-void OdpGenerator::setDocumentMetaData(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::setDocumentMetaData(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->setDocumentMetaData(propList);
 }
 
-void OdpGenerator::defineEmbeddedFont(const librevenge::RVNGPropertyList &/*propList*/)
+void OdpGenerator::defineEmbeddedFont(const librevenge::RVNGPropertyList &propList)
 {
-	// TODO: implement me
+	mpImpl->defineEmbeddedFont(propList);
 }
 
-void OdpGenerator::startSlide(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::startSlide(const librevenge::RVNGPropertyList &propList)
 {
 	librevenge::RVNGPropertyList pList(propList);
 
-	mpImpl->mpCurrentPageSpan=0;
-	if (pList["librevenge:master-page-name"])
-	{
-		mpImpl->mpCurrentPageSpan=mpImpl->getPageSpanManager().get(pList["librevenge:master-page-name"]->getStr());
-		if (!mpImpl->mpCurrentPageSpan)
-			pList.remove("librevenge:master-page-name");
-	}
-
+	mpImpl->mpCurrentPageSpan=nullptr;
 	if (!mpImpl->mpCurrentPageSpan)
 	{
 		mpImpl->updatePageSpanPropertiesToCreatePage(pList);
@@ -570,7 +591,7 @@ void OdpGenerator::startSlide(const ::librevenge::RVNGPropertyList &propList)
 		pageName.appendEscapedXML(propList["draw:name"]->getStr());
 	else
 		pageName.sprintf("page%i", mpImpl->miPageIndex);
-	TagOpenElement *pDrawPageOpenElement = new TagOpenElement("draw:page");
+	auto pDrawPageOpenElement = std::make_shared<TagOpenElement>("draw:page");
 	pDrawPageOpenElement->addAttribute("draw:name", pageName);
 	pDrawPageOpenElement->addAttribute("draw:style-name", mpImpl->mpCurrentPageSpan->getDrawingName());
 	pDrawPageOpenElement->addAttribute("draw:master-page-name", mpImpl->mpCurrentPageSpan->getMasterName());
@@ -579,10 +600,10 @@ void OdpGenerator::startSlide(const ::librevenge::RVNGPropertyList &propList)
 
 void OdpGenerator::endSlide()
 {
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:page"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:page"));
 }
 
-void OdpGenerator::startMasterSlide(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::startMasterSlide(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->inMasterPage())
 	{
@@ -599,7 +620,7 @@ void OdpGenerator::startMasterSlide(const ::librevenge::RVNGPropertyList &propLi
 		PageSpan *pageSpan=mpImpl->getPageSpanManager().add(pList, true);
 		if (pageSpan)
 		{
-			libodfgen::DocumentElementVector *pMasterElements = new libodfgen::DocumentElementVector;
+			auto pMasterElements = std::make_shared<libodfgen::DocumentElementVector>();
 			pageSpan->setMasterContent(pMasterElements);
 			mpImpl->pushStorage(pMasterElements);
 		}
@@ -609,7 +630,7 @@ void OdpGenerator::startMasterSlide(const ::librevenge::RVNGPropertyList &propLi
 	if (!ok)
 	{
 		ODFGEN_DEBUG_MSG(("OdpGenerator::startMasterSlide creation of the master page has failed\n"));
-		mpImpl->pushStorage(&mpImpl->mDummyMasterSlideStorage);
+		mpImpl->pushStorage(std::shared_ptr<libodfgen::DocumentElementVector>(&mpImpl->mDummyMasterSlideStorage, libodfgen::DummyDeleter()));
 	}
 }
 
@@ -625,17 +646,17 @@ void OdpGenerator::endMasterSlide()
 	mpImpl->mDummyMasterSlideStorage.clear();
 }
 
-void OdpGenerator::setStyle(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::setStyle(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->defineGraphicStyle(propList);
 }
 
-void OdpGenerator::setSlideTransition(const ::librevenge::RVNGPropertyList &/*propList*/)
+void OdpGenerator::setSlideTransition(const librevenge::RVNGPropertyList &/*propList*/)
 {
 	// TODO: implement me
 }
 
-void OdpGenerator::startLayer(const ::librevenge::RVNGPropertyList & /* propList */)
+void OdpGenerator::startLayer(const librevenge::RVNGPropertyList & /* propList */)
 {
 }
 
@@ -643,22 +664,22 @@ void OdpGenerator::endLayer()
 {
 }
 
-void OdpGenerator::drawRectangle(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::drawRectangle(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawRectangle(propList);
 }
 
-void OdpGenerator::drawEllipse(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::drawEllipse(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawEllipse(propList);
 }
 
-void OdpGenerator::drawPolyline(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::drawPolyline(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawPolySomething(propList, false);
 }
 
-void OdpGenerator::drawPolygon(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::drawPolygon(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawPolySomething(propList, true);
 }
@@ -668,12 +689,15 @@ void OdpGenerator::drawPath(const librevenge::RVNGPropertyList &propList)
 	mpImpl->drawPath(propList);
 }
 
-void OdpGenerator::drawGraphicObject(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::drawGraphicObject(const librevenge::RVNGPropertyList &propList)
 {
-	if (!propList["librevenge:mime-type"] || propList["librevenge:mime-type"]->getStr().len() <= 0)
-		return;
-	if (!propList["office:binary-data"])
-		return;
+	if (!propList["librevenge:xlink"] || propList["librevenge:xlink"]->getStr().empty())
+	{
+		if (!propList["librevenge:mime-type"] || propList["librevenge:mime-type"]->getStr().len() <= 0)
+			return;
+		if (!propList["office:binary-data"])
+			return;
+	}
 	if (!propList["svg:x"] || !propList["svg:y"] || !propList["svg:width"] || !propList["svg:height"])
 		return;
 
@@ -685,6 +709,8 @@ void OdpGenerator::drawGraphicObject(const ::librevenge::RVNGPropertyList &propL
 		style.insert("style:mirror", "horizontal");
 	else
 		style.insert("style:mirror", "none");
+	if (propList["fo:clip"])
+		style.insert("fo:clip", propList["fo:clip"]->getStr());
 	if (propList["draw:color-mode"])
 		style.insert("draw:color-mode", propList["draw:color-mode"]->getStr());
 	if (propList["draw:luminance"])
@@ -716,7 +742,7 @@ void OdpGenerator::drawGraphicObject(const ::librevenge::RVNGPropertyList &propL
 	}
 
 	double angle(propList["librevenge:rotate"] ? - M_PI * propList["librevenge:rotate"]->getDouble() / 180.0 : 0.0);
-	if (angle != 0.0)
+	if (angle < 0 || angle > 0)
 	{
 		double deltax((width*cos(angle)+height*sin(angle)-width)/2.0);
 		double deltay((-width*sin(angle)+height*cos(angle)-height)/2.0);
@@ -731,7 +757,7 @@ void OdpGenerator::drawGraphicObject(const ::librevenge::RVNGPropertyList &propL
 	framePropList.insert("svg:height", height);
 	framePropList.insert("svg:width", width);
 
-	TagOpenElement *pDrawFrameElement = new TagOpenElement("draw:frame");
+	auto pDrawFrameElement = std::make_shared<TagOpenElement>("draw:frame");
 
 	librevenge::RVNGPropertyList finalStyle;
 	mpImpl->getGraphicManager().addGraphicProperties(style, finalStyle);
@@ -742,7 +768,7 @@ void OdpGenerator::drawGraphicObject(const ::librevenge::RVNGPropertyList &propL
 	pDrawFrameElement->addAttribute("svg:height", framePropList["svg:height"]->getStr());
 	pDrawFrameElement->addAttribute("svg:width", framePropList["svg:width"]->getStr());
 
-	if (angle != 0.0)
+	if (angle < 0 || angle > 0)
 	{
 		framePropList.insert("librevenge:rotate", angle, librevenge::RVNG_GENERIC);
 		librevenge::RVNGString sValue;
@@ -761,10 +787,10 @@ void OdpGenerator::drawGraphicObject(const ::librevenge::RVNGPropertyList &propL
 
 	mpImpl->insertBinaryObject(propList);
 
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:frame"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:frame"));
 }
 
-void OdpGenerator::drawConnector(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::drawConnector(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawConnector(propList);
 }
@@ -777,7 +803,7 @@ void OdpGenerator::endEmbeddedGraphics()
 {
 }
 
-void OdpGenerator::openGroup(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::openGroup(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->openGroup(propList);
 }
@@ -790,7 +816,7 @@ void OdpGenerator::closeGroup()
 void OdpGenerator::startTextObject(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->openTextBoxFrame(propList);
-	mpImpl->getCurrentStorage()->push_back(new TagOpenElement("draw:text-box"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("draw:text-box"));
 	mpImpl->mState.mbIsTextBox = true;
 	mpImpl->pushListState();
 }
@@ -802,7 +828,7 @@ void OdpGenerator::endTextObject()
 
 	mpImpl->popListState();
 	mpImpl->mState.mbIsTextBox = false;
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:text-box"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:text-box"));
 	mpImpl->closeTextBoxFrame();
 }
 
@@ -866,17 +892,17 @@ void OdpGenerator::insertLineBreak()
 	mpImpl->insertLineBreak();
 }
 
-void OdpGenerator::insertField(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::insertField(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->insertField(propList);
 }
 
-void OdpGenerator::openOrderedListLevel(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::openOrderedListLevel(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->openListLevel(propList, true);
 }
 
-void OdpGenerator::openUnorderedListLevel(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::openUnorderedListLevel(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->openListLevel(propList, false);
 }
@@ -891,7 +917,7 @@ void OdpGenerator::closeUnorderedListLevel()
 	mpImpl->closeListLevel();
 }
 
-void OdpGenerator::openListElement(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::openListElement(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->openListElement(propList);
 }
@@ -901,14 +927,14 @@ void OdpGenerator::closeListElement()
 	mpImpl->closeListElement();
 }
 
-void OdpGenerator::startTableObject(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::startTableObject(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->mState.mInComment)
 		return;
 	mpImpl->pushListState();
 
 	// table must be inside a frame
-	TagOpenElement *pFrameOpenElement = new TagOpenElement("draw:frame");
+	auto pFrameOpenElement = std::make_shared<TagOpenElement>("draw:frame");
 
 	pFrameOpenElement->addAttribute("draw:style-name", "standard");
 	if (propList["svg:x"])
@@ -929,11 +955,11 @@ void OdpGenerator::endTableObject()
 	if (mpImpl->mState.mInComment)
 		return;
 	mpImpl->closeTable();
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:frame"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:frame"));
 	mpImpl->popListState();
 }
 
-void OdpGenerator::openTableRow(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::openTableRow(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->mState.mInComment)
 		return;
@@ -947,7 +973,7 @@ void OdpGenerator::closeTableRow()
 	mpImpl->closeTableRow();
 }
 
-void OdpGenerator::openTableCell(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::openTableCell(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->mState.mInComment)
 		return;
@@ -988,7 +1014,7 @@ void OdpGenerator::closeTableCell()
 	mpImpl->mState.mTableCellOpened = false;
 }
 
-void OdpGenerator::insertCoveredTableCell(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::insertCoveredTableCell(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->mState.mInComment)
 		return;
@@ -996,7 +1022,7 @@ void OdpGenerator::insertCoveredTableCell(const ::librevenge::RVNGPropertyList &
 	mpImpl->insertCoveredTableCell(propList);
 }
 
-void OdpGenerator::startComment(const ::librevenge::RVNGPropertyList &propList)
+void OdpGenerator::startComment(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->mState.mInComment)
 	{
@@ -1006,7 +1032,7 @@ void OdpGenerator::startComment(const ::librevenge::RVNGPropertyList &propList)
 	mpImpl->pushListState();
 	mpImpl->mState.mInComment = true;
 
-	TagOpenElement *const commentElement = new TagOpenElement("officeooo:annotation");
+	auto const commentElement = std::make_shared<TagOpenElement>("officeooo:annotation");
 
 	// position & size
 	double val;
@@ -1031,10 +1057,10 @@ void OdpGenerator::endComment()
 	}
 	mpImpl->popListState();
 	mpImpl->mState.mInComment = false;
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("officeooo:annotation"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("officeooo:annotation"));
 }
 
-void OdpGenerator::startNotes(const ::librevenge::RVNGPropertyList &/*propList*/)
+void OdpGenerator::startNotes(const librevenge::RVNGPropertyList &/*propList*/)
 {
 	if (mpImpl->mState.mInNotes)
 	{
@@ -1043,12 +1069,12 @@ void OdpGenerator::startNotes(const ::librevenge::RVNGPropertyList &/*propList*/
 	}
 
 	mpImpl->pushListState();
-	TagOpenElement *const notesElement = new TagOpenElement("presentation:notes");
+	auto const notesElement = std::make_shared<TagOpenElement>("presentation:notes");
 	notesElement->addAttribute("draw:style-name", "PresentationNotesPage");
 
 	mpImpl->getCurrentStorage()->push_back(notesElement);
 
-	TagOpenElement *const thumbnailElement = new TagOpenElement("draw:page-thumbnail");
+	auto const thumbnailElement = std::make_shared<TagOpenElement>("draw:page-thumbnail");
 	thumbnailElement->addAttribute("draw:layer", "layout");
 	thumbnailElement->addAttribute("presentation:class", "page");
 	// TODO: should the dimensions be hardcoded? If not, where
@@ -1062,9 +1088,9 @@ void OdpGenerator::startNotes(const ::librevenge::RVNGPropertyList &/*propList*/
 	thumbnailElement->addAttribute("draw:page-number", pageNumber);
 
 	mpImpl->getCurrentStorage()->push_back(thumbnailElement);
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:page-thumbnail"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:page-thumbnail"));
 
-	TagOpenElement *const frameElement = new TagOpenElement("draw:frame");
+	auto const frameElement = std::make_shared<TagOpenElement>("draw:frame");
 	frameElement->addAttribute("presentation:style-name", "PresentationNotesFrame");
 	frameElement->addAttribute("draw:layer", "layout");
 	frameElement->addAttribute("presentation:class", "notes");
@@ -1075,7 +1101,7 @@ void OdpGenerator::startNotes(const ::librevenge::RVNGPropertyList &/*propList*/
 
 	mpImpl->getCurrentStorage()->push_back(frameElement);
 
-	mpImpl->getCurrentStorage()->push_back(new TagOpenElement("draw:text-box"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("draw:text-box"));
 
 	mpImpl->mState.mInNotes = true;
 }
@@ -1090,17 +1116,17 @@ void OdpGenerator::endNotes()
 	mpImpl->popListState();
 	mpImpl->mState.mInNotes = false;
 
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:text-box"));
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:frame"));
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("presentation:notes"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:text-box"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:frame"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("presentation:notes"));
 }
 
-void OdpGenerator::defineChartStyle(const ::librevenge::RVNGPropertyList &)
+void OdpGenerator::defineChartStyle(const librevenge::RVNGPropertyList &)
 {
 	// TODO: implement me
 }
 
-void OdpGenerator::openChart(const ::librevenge::RVNGPropertyList &)
+void OdpGenerator::openChart(const librevenge::RVNGPropertyList &)
 {
 	// TODO: implement me
 }
@@ -1110,7 +1136,7 @@ void OdpGenerator::closeChart()
 	// TODO: implement me
 }
 
-void OdpGenerator::openChartTextObject(const ::librevenge::RVNGPropertyList &)
+void OdpGenerator::openChartTextObject(const librevenge::RVNGPropertyList &)
 {
 	// TODO: implement me
 }
@@ -1120,7 +1146,7 @@ void OdpGenerator::closeChartTextObject()
 	// TODO: implement me
 }
 
-void OdpGenerator::openChartPlotArea(const ::librevenge::RVNGPropertyList &)
+void OdpGenerator::openChartPlotArea(const librevenge::RVNGPropertyList &)
 {
 	// TODO: implement me
 }
@@ -1130,12 +1156,12 @@ void OdpGenerator::closeChartPlotArea()
 	// TODO: implement me
 }
 
-void OdpGenerator::insertChartAxis(const ::librevenge::RVNGPropertyList &)
+void OdpGenerator::insertChartAxis(const librevenge::RVNGPropertyList &)
 {
 	// TODO: implement me
 }
 
-void OdpGenerator::openChartSeries(const ::librevenge::RVNGPropertyList &)
+void OdpGenerator::openChartSeries(const librevenge::RVNGPropertyList &)
 {
 	// TODO: implement me
 }
@@ -1145,7 +1171,7 @@ void OdpGenerator::closeChartSeries()
 	// TODO: implement me
 }
 
-void OdpGenerator::openAnimationSequence(const ::librevenge::RVNGPropertyList &/*propList*/)
+void OdpGenerator::openAnimationSequence(const librevenge::RVNGPropertyList &/*propList*/)
 {
 	// TODO: implement me
 }
@@ -1155,7 +1181,7 @@ void OdpGenerator::closeAnimationSequence()
 	// TODO: implement me
 }
 
-void OdpGenerator::openAnimationGroup(const ::librevenge::RVNGPropertyList &/*propList*/)
+void OdpGenerator::openAnimationGroup(const librevenge::RVNGPropertyList &/*propList*/)
 {
 	// TODO: implement me
 }
@@ -1165,7 +1191,7 @@ void OdpGenerator::closeAnimationGroup()
 	// TODO: implement me
 }
 
-void OdpGenerator::openAnimationIteration(const ::librevenge::RVNGPropertyList &/*propList*/)
+void OdpGenerator::openAnimationIteration(const librevenge::RVNGPropertyList &/*propList*/)
 {
 	// TODO: implement me
 }
@@ -1175,22 +1201,22 @@ void OdpGenerator::closeAnimationIteration()
 	// TODO: implement me
 }
 
-void OdpGenerator::insertMotionAnimation(const ::librevenge::RVNGPropertyList &/*propList*/)
+void OdpGenerator::insertMotionAnimation(const librevenge::RVNGPropertyList &/*propList*/)
 {
 	// TODO: implement me
 }
 
-void OdpGenerator::insertColorAnimation(const ::librevenge::RVNGPropertyList &/*propList*/)
+void OdpGenerator::insertColorAnimation(const librevenge::RVNGPropertyList &/*propList*/)
 {
 	// TODO: implement me
 }
 
-void OdpGenerator::insertAnimation(const ::librevenge::RVNGPropertyList &/*propList*/)
+void OdpGenerator::insertAnimation(const librevenge::RVNGPropertyList &/*propList*/)
 {
 	// TODO: implement me
 }
 
-void OdpGenerator::insertEffect(const ::librevenge::RVNGPropertyList &/*propList*/)
+void OdpGenerator::insertEffect(const librevenge::RVNGPropertyList &/*propList*/)
 {
 	// TODO: implement me
 }
