@@ -24,15 +24,17 @@
  * Corel Corporation or Corel Corporation Limited."
  */
 
+#include <libodfgen/libodfgen.hxx>
+
 #include <librevenge/librevenge.h>
 
+#include <iomanip>
+#include <locale>
 #include <map>
 #include <stack>
 #include <sstream>
 #include <string>
 
-#include <libodfgen/libodfgen.hxx>
-
 #include "DocumentElement.hxx"
 #include "FilterInternal.hxx"
 
@@ -46,6 +48,19 @@
 #include "OdcGenerator.hxx"
 #include "OdfGenerator.hxx"
 
+namespace
+{
+
+librevenge::RVNGString makePreciseStr(const double value)
+{
+	std::ostringstream os;
+	os.imbue(std::locale::classic());
+	os << std::fixed << std::setprecision(8) << value;
+	return os.str().c_str();
+}
+
+}
+
 class OdsGeneratorPrivate : public OdfGenerator
 {
 public:
@@ -95,7 +110,11 @@ public:
 	// the odc state
 	struct OdcGeneratorState
 	{
-		OdcGeneratorState(librevenge::RVNGString const &dir) : mDir(dir), mContentElements(), mInternalHandler(&mContentElements), mGenerator()
+		explicit OdcGeneratorState(librevenge::RVNGString const &dir)
+			: mDir(dir)
+			, mContentElements()
+			, mInternalHandler(&mContentElements)
+			, mGenerator()
 		{
 			if (!mDir.empty()) return;
 			mGenerator.addDocumentHandler(&mInternalHandler,ODF_FLAT_XML);
@@ -133,7 +152,7 @@ public:
 	};
 
 	OdsGeneratorPrivate();
-	~OdsGeneratorPrivate();
+	~OdsGeneratorPrivate() override;
 	void addDocumentHandler(OdfDocumentHandler *pHandler, const OdfStreamType streamType)
 	{
 		if (!pHandler)
@@ -210,7 +229,7 @@ public:
 		}
 		if (add && !state.mbInSheetShapes)
 		{
-			getCurrentStorage()->push_back(new TagOpenElement("table:shapes"));
+			getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("table:shapes"));
 			state.mbInSheetShapes=true;
 		}
 		return true;
@@ -265,7 +284,7 @@ public:
 			ODFGEN_DEBUG_MSG(("OdsGeneratorPrivate::sendAuxiliarOdcGenerator: data seems bad\n"));
 			return false;
 		}
-		TagOpenElement *object=new TagOpenElement("draw:object");
+		auto object=std::make_shared<TagOpenElement>("draw:object");
 		if (!mAuxiliarOdcState->mDir.empty())
 		{
 			object->addAttribute("xlink:href",mAuxiliarOdcState->mDir.cstr());
@@ -274,9 +293,9 @@ public:
 			object->addAttribute("xlink:actuate","onLoad");
 		}
 		getCurrentStorage()->push_back(object);
-		mAuxiliarOdcState->mContentElements.appendTo(*getCurrentStorage());
+		getCurrentStorage()->insert(getCurrentStorage()->end(), mAuxiliarOdcState->mContentElements.begin(), mAuxiliarOdcState->mContentElements.end());
 		mAuxiliarOdcState->mContentElements.resize(0);
-		getCurrentStorage()->push_back(new TagCloseElement("draw:object"));
+		getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:object"));
 		return true;
 	}
 	void resetAuxiliarOdcGenerator()
@@ -324,10 +343,10 @@ public:
 			ODFGEN_DEBUG_MSG(("OdsGeneratorPrivate::sendAuxiliarOdtGenerator: data seems bad\n"));
 			return false;
 		}
-		getCurrentStorage()->push_back(new TagOpenElement("draw:object"));
-		mAuxiliarOdtState->mContentElements.appendTo(*getCurrentStorage());
+		getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("draw:object"));
+		getCurrentStorage()->insert(getCurrentStorage()->end(), mAuxiliarOdtState->mContentElements.begin(), mAuxiliarOdtState->mContentElements.end());
 		mAuxiliarOdtState->mContentElements.resize(0);
-		getCurrentStorage()->push_back(new TagCloseElement("draw:object"));
+		getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:object"));
 		return true;
 	}
 	void resetAuxiliarOdtGenerator()
@@ -342,7 +361,7 @@ public:
 		return false;
 	}
 
-	bool writeTargetDocument(OdfDocumentHandler *pHandler, OdfStreamType streamType);
+	bool writeTargetDocument(OdfDocumentHandler *pHandler, OdfStreamType streamType) override;
 	void _writeStyles(OdfDocumentHandler *pHandler);
 	void _writeAutomaticStyles(OdfDocumentHandler *pHandler, OdfStreamType streamType);
 	void initPageManager();
@@ -351,9 +370,9 @@ public:
 	std::stack<State> mStateStack;
 
 	// auxiliar odc handler to create data
-	shared_ptr<OdcGeneratorState> mAuxiliarOdcState;
+	std::shared_ptr<OdcGeneratorState> mAuxiliarOdcState;
 	// auxiliar odt handler to create data
-	shared_ptr<OdtGeneratorState> mAuxiliarOdtState;
+	std::shared_ptr<OdtGeneratorState> mAuxiliarOdtState;
 
 	// table styles
 	SheetManager  mSheetManager;
@@ -372,8 +391,8 @@ OdsGeneratorPrivate::OdsGeneratorPrivate() : OdfGenerator(),
 	mCommandStack(),
 	mStateStack(),
 	mAuxiliarOdcState(), mAuxiliarOdtState(),
-	mSheetManager(),
-	mpCurrentPageSpan(0)
+	mSheetManager(mNumberingManager),
+	mpCurrentPageSpan(nullptr)
 {
 	mStateStack.push(State());
 	initPageManager();
@@ -393,7 +412,7 @@ bool OdsGeneratorPrivate::close(Command command)
 	if (mCommandStack.empty() || mCommandStack.top()!=command)
 	{
 #ifdef DEBUG
-		static char const *(wh[]) =
+		static char const *wh[] =
 		{
 			"Document", "PageSpan", "Header", "Footer", "Sheet", "SheetRow", "SheetCell",
 			"Chart", "ChartDataLabel", "ChartPlotArea", "ChartSerie", "ChartTextObject",
@@ -485,6 +504,7 @@ void OdsGeneratorPrivate::_writeAutomaticStyles(OdfDocumentHandler *pHandler, Od
 		mParagraphManager.write(pHandler, Style::Z_StyleAutomatic);
 		mListManager.write(pHandler, Style::Z_StyleAutomatic);
 		mGraphicManager.write(pHandler, Style::Z_StyleAutomatic);
+		mNumberingManager.write(pHandler, Style::Z_StyleAutomatic);
 		mSheetManager.write(pHandler, Style::Z_StyleAutomatic);
 	}
 
@@ -495,6 +515,7 @@ void OdsGeneratorPrivate::_writeAutomaticStyles(OdfDocumentHandler *pHandler, Od
 		mParagraphManager.write(pHandler, Style::Z_ContentAutomatic);
 		mListManager.write(pHandler, Style::Z_ContentAutomatic);
 		mGraphicManager.write(pHandler, Style::Z_ContentAutomatic);
+		mNumberingManager.write(pHandler, Style::Z_ContentAutomatic);
 		mSheetManager.write(pHandler, Style::Z_ContentAutomatic);
 	}
 
@@ -572,7 +593,7 @@ void OdsGeneratorPrivate::_writeStyles(OdfDocumentHandler *pHandler)
 	standardStyleOpenElement.write(pHandler);
 	pHandler->endElement("style:style");
 
-	static char const *(s_paraStyle[4*10]) =
+	static char const *s_paraStyle[4*10] =
 	{
 		"Text_Body", "Text Body", "Standard", "text",
 		"Table_Contents", "Table Contents", "Text_Body", "extra",
@@ -597,7 +618,7 @@ void OdsGeneratorPrivate::_writeStyles(OdfDocumentHandler *pHandler)
 		pHandler->endElement("style:style");
 	}
 
-	static char const *(s_textStyle[2*4])=
+	static char const *s_textStyle[2*4] =
 	{
 		"Footnote_Symbol", "Footnote Symbol", "Endnote_Symbol", "Endnote Symbol",
 		"Footnote_anchor", "Footnote anchor", "Endnote_anchor", "Endnote anchor"
@@ -619,6 +640,7 @@ void OdsGeneratorPrivate::_writeStyles(OdfDocumentHandler *pHandler)
 	mSpanManager.write(pHandler, Style::Z_Style);
 	mParagraphManager.write(pHandler, Style::Z_Style);
 	mListManager.write(pHandler, Style::Z_Style);
+	mFillManager.write(pHandler);
 	mGraphicManager.write(pHandler, Style::Z_Style);
 	pHandler->endElement("office:styles");
 }
@@ -714,7 +736,7 @@ bool OdsGeneratorPrivate::writeTargetDocument(OdfDocumentHandler *pHandler, OdfS
 		// writing out the document
 		TagOpenElement("office:body").write(pHandler);
 		TagOpenElement("office:spreadsheet").write(pHandler);
-		sendStorage(&mBodyStorage, pHandler);
+		sendStorage(mpBodyStorage.get(), pHandler);
 		pHandler->endElement("office:spreadsheet");
 		pHandler->endElement("office:body");
 		ODFGEN_DEBUG_MSG(("OdsGenerator: Document Body: Finished writing all doc els..\n"));
@@ -732,9 +754,9 @@ void OdsGenerator::setDocumentMetaData(const librevenge::RVNGPropertyList &propL
 	mpImpl->setDocumentMetaData(propList);
 }
 
-void OdsGenerator::defineEmbeddedFont(const librevenge::RVNGPropertyList &/*propList*/)
+void OdsGenerator::defineEmbeddedFont(const librevenge::RVNGPropertyList &propList)
 {
-	// TODO: implement me
+	mpImpl->defineEmbeddedFont(propList);
 }
 
 void OdsGenerator::openPageSpan(const librevenge::RVNGPropertyList &propList)
@@ -752,12 +774,7 @@ void OdsGenerator::closePageSpan()
 
 void OdsGenerator::defineSheetNumberingStyle(const librevenge::RVNGPropertyList &propList)
 {
-	if (!mpImpl->getState().mbInSheet || !mpImpl->mSheetManager.actualSheet())
-	{
-		ODFGEN_DEBUG_MSG(("OdsGenerator::defineSheetNumberingStyle can not be called outside a sheet!!!\n"));
-		return;
-	}
-	mpImpl->mSheetManager.actualSheet()->addNumberingStyle(propList);
+	mpImpl->getNumberingManager().addStyle(propList);
 }
 
 void OdsGenerator::openSheet(const librevenge::RVNGPropertyList &propList)
@@ -776,7 +793,7 @@ void OdsGenerator::openSheet(const librevenge::RVNGPropertyList &propList)
 	}
 
 	librevenge::RVNGPropertyList finalPropList(propList);
-	if (mpImpl->getCurrentStorage() == &mpImpl->getBodyStorage())
+	if (mpImpl->getCurrentStorage() == mpImpl->getBodyStorage())
 	{
 		if (mpImpl->mpCurrentPageSpan)
 			finalPropList.insert("style:master-page-name", mpImpl->mpCurrentPageSpan->getMasterName());
@@ -792,7 +809,7 @@ void OdsGenerator::openSheet(const librevenge::RVNGPropertyList &propList)
 	SheetStyle *style=mpImpl->mSheetManager.actualSheet();
 	if (!style) return;
 	librevenge::RVNGString sTableName(style->getName());
-	TagOpenElement *pTableOpenElement = new TagOpenElement("table:table");
+	auto pTableOpenElement = std::make_shared<TagOpenElement>("table:table");
 	if (propList["librevenge:sheet-name"])
 		pTableOpenElement->addAttribute("table:name", propList["librevenge:sheet-name"]->getStr());
 	else
@@ -803,17 +820,7 @@ void OdsGenerator::openSheet(const librevenge::RVNGPropertyList &propList)
 	/* TODO: open a table:shapes element
 	   a environment to store the table content which must be merged in closeSheet
 	*/
-	for (int i=0; i< style->getNumColumns(); ++i)
-	{
-		TagOpenElement *pTableColumnOpenElement = new TagOpenElement("table:table-column");
-		librevenge::RVNGString sColumnStyleName;
-		sColumnStyleName.sprintf("%s_col%i", sTableName.cstr(), (i+1));
-		pTableColumnOpenElement->addAttribute("table:style-name", sColumnStyleName.cstr());
-		mpImpl->getCurrentStorage()->push_back(pTableColumnOpenElement);
-
-		TagCloseElement *pTableColumnCloseElement = new TagCloseElement("table:table-column");
-		mpImpl->getCurrentStorage()->push_back(pTableColumnCloseElement);
-	}
+	style->addColumnDefinitions(*mpImpl->getCurrentStorage());
 }
 
 void OdsGenerator::closeSheet()
@@ -825,11 +832,11 @@ void OdsGenerator::closeSheet()
 	if (mpImpl->mAuxiliarOdcState || mpImpl->mAuxiliarOdtState || !state.mbInSheet) return;
 	if (state.mbInSheetShapes)
 	{
-		mpImpl->getCurrentStorage()->push_back(new TagCloseElement("table:shapes"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("table:shapes"));
 		mpImpl->getState().mbInSheetShapes=false;
 	}
 	mpImpl->mSheetManager.closeSheet();
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("table:table"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("table:table"));
 }
 
 void OdsGenerator::openSheetRow(const librevenge::RVNGPropertyList &propList)
@@ -846,7 +853,7 @@ void OdsGenerator::openSheetRow(const librevenge::RVNGPropertyList &propList)
 	}
 	if (state.mbInSheetShapes)
 	{
-		mpImpl->getCurrentStorage()->push_back(new TagCloseElement("table:shapes"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("table:shapes"));
 		mpImpl->getState().mbInSheetShapes=false;
 	}
 
@@ -856,28 +863,33 @@ void OdsGenerator::openSheetRow(const librevenge::RVNGPropertyList &propList)
 	{
 		librevenge::RVNGString sEmptyRowStyleName=style->addRow(librevenge::RVNGPropertyList());
 
-		TagOpenElement *pEmptyRowOpenElement = new TagOpenElement("table:table-row");
+		auto pEmptyRowOpenElement = std::make_shared<TagOpenElement>("table:table-row");
 		pEmptyRowOpenElement->addAttribute("table:style-name", sEmptyRowStyleName);
 		librevenge::RVNGString numEmpty;
 		numEmpty.sprintf("%d", row-state.miLastSheetRow);
 		pEmptyRowOpenElement->addAttribute("table:number-rows-repeated", numEmpty);
 
 		mpImpl->getCurrentStorage()->push_back(pEmptyRowOpenElement);
-		mpImpl->getCurrentStorage()->push_back(new TagOpenElement("table:table-cell"));
-		mpImpl->getCurrentStorage()->push_back(new TagCloseElement("table:table-cell"));
-		mpImpl->getCurrentStorage()->push_back(new TagCloseElement("table:table-row"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("table:table-cell"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("table:table-cell"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("table:table-row"));
 	}
 	else
 		row=state.miLastSheetRow;
-	mpImpl->getState().miLastSheetRow=row+1;
+	int numRows=1;
+	if (propList["table:number-rows-repeated"] && propList["table:number-rows-repeated"]->getInt()>1)
+		numRows=propList["table:number-rows-repeated"]->getInt();
+	mpImpl->getState().miLastSheetRow=row+numRows;
 
 	state.miLastSheetColumn=0;
 	state.mbInSheetRow=state.mbFirstInSheetRow=true;
 	mpImpl->pushState(state);
 
 	librevenge::RVNGString sSheetRowStyleName=style->addRow(propList);
-	TagOpenElement *pSheetRowOpenElement = new TagOpenElement("table:table-row");
+	auto pSheetRowOpenElement = std::make_shared<TagOpenElement>("table:table-row");
 	pSheetRowOpenElement->addAttribute("table:style-name", sSheetRowStyleName);
+	if (numRows>1)
+		pSheetRowOpenElement->addAttribute("table:number-rows-repeated", propList["table:number-rows-repeated"]->getStr());
 	mpImpl->getCurrentStorage()->push_back(pSheetRowOpenElement);
 }
 
@@ -888,13 +900,13 @@ void OdsGenerator::closeSheetRow()
 	if (!state.mbInSheetRow) return;
 	if (state.mbFirstInSheetRow)
 	{
-		TagOpenElement *pSheetCellOpenElement = new TagOpenElement("table:table-cell");
+		auto pSheetCellOpenElement = std::make_shared<TagOpenElement>("table:table-cell");
 		pSheetCellOpenElement->addAttribute("table:number-columns-repeated","1");
 		mpImpl->getCurrentStorage()->push_back(pSheetCellOpenElement);
-		mpImpl->getCurrentStorage()->push_back(new TagCloseElement("table:table-cell"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("table:table-cell"));
 	}
 	mpImpl->popState();
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("table:table-row"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("table:table-row"));
 }
 
 void OdsGenerator::openSheetCell(const librevenge::RVNGPropertyList &propList)
@@ -914,16 +926,19 @@ void OdsGenerator::openSheetCell(const librevenge::RVNGPropertyList &propList)
 	int col = propList["librevenge:column"] ? propList["librevenge:column"]->getInt() : -1;
 	if (col > state.miLastSheetColumn)
 	{
-		TagOpenElement *pEmptyElement = new TagOpenElement("table:table-cell");
+		auto pEmptyElement = std::make_shared<TagOpenElement>("table:table-cell");
 		librevenge::RVNGString numEmpty;
 		numEmpty.sprintf("%d", col-state.miLastSheetColumn);
 		pEmptyElement->addAttribute("table:number-columns-repeated", numEmpty);
 		mpImpl->getCurrentStorage()->push_back(pEmptyElement);
-		mpImpl->getCurrentStorage()->push_back(new TagCloseElement("table:table-cell"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("table:table-cell"));
 	}
 	else
 		col=state.miLastSheetColumn;
-	mpImpl->getState().miLastSheetColumn=col+1;
+	int numColumns=1;
+	if (propList["table:number-columns-repeated"] && propList["table:number-columns-repeated"]->getInt()>1)
+		numColumns=propList["table:number-columns-repeated"]->getInt();
+	mpImpl->getState().miLastSheetColumn=col+numColumns;
 
 	state.mbInSheetCell=true;
 	mpImpl->pushState(state);
@@ -932,8 +947,22 @@ void OdsGenerator::openSheetCell(const librevenge::RVNGPropertyList &propList)
 		mpImpl->getFontManager().findOrAdd(propList["style:font-name"]->getStr().cstr());
 	librevenge::RVNGString sSheetCellStyleName=style->addCell(propList);
 
-	TagOpenElement *pSheetCellOpenElement = new TagOpenElement("table:table-cell");
+	auto pSheetCellOpenElement = std::make_shared<TagOpenElement>("table:table-cell");
 	pSheetCellOpenElement->addAttribute("table:style-name", sSheetCellStyleName);
+	if (propList["table:number-columns-spanned"] && propList["table:number-columns-spanned"]->getInt()>1)
+		pSheetCellOpenElement->addAttribute("table:number-columns-spanned",
+		                                    propList["table:number-columns-spanned"]->getStr().cstr());
+	if (propList["table:number-rows-spanned"] && propList["table:number-rows-spanned"]->getInt()>1)
+		pSheetCellOpenElement->addAttribute("table:number-rows-spanned",
+		                                    propList["table:number-rows-spanned"]->getStr().cstr());
+	if (numColumns>1)
+		pSheetCellOpenElement->addAttribute("table:number-columns-repeated", propList["table:number-columns-repeated"]->getStr());
+	if (propList["table:number-matrix-columns-spanned"] && propList["table:number-matrix-columns-spanned"]->getInt()>1)
+		pSheetCellOpenElement->addAttribute("table:number-matrix-columns-spanned",
+		                                    propList["table:number-matrix-columns-spanned"]->getStr().cstr());
+	if (propList["table:number-matrix-rows-spanned"] && propList["table:number-matrix-rows-spanned"]->getInt()>1)
+		pSheetCellOpenElement->addAttribute("table:number-matrix-rows-spanned",
+		                                    propList["table:number-matrix-rows-spanned"]->getStr().cstr());
 
 	if (propList["librevenge:value-type"])
 	{
@@ -955,8 +984,19 @@ void OdsGenerator::openSheetCell(const librevenge::RVNGPropertyList &propList)
 				}
 				else
 				{
+					// we need the maximum precision here, so we must avoid getStr() when possible
+					librevenge::RVNGString value;
+					if (propList["librevenge:value"]->getUnit()==librevenge::RVNG_GENERIC)
+						value = makePreciseStr(propList["librevenge:value"]->getDouble());
+					else if (propList["librevenge:value"]->getUnit()==librevenge::RVNG_PERCENT)
+					{
+						value = makePreciseStr(propList["librevenge:value"]->getDouble()*100.);
+						value.append('%');
+					}
+					else
+						value=propList["librevenge:value"]->getStr();
 					pSheetCellOpenElement->addAttribute("office:value-type", valueType.c_str());
-					pSheetCellOpenElement->addAttribute("office:value", propList["librevenge:value"]->getStr().cstr());
+					pSheetCellOpenElement->addAttribute("office:value", value.cstr());
 				}
 			}
 		}
@@ -1027,7 +1067,7 @@ void OdsGenerator::closeSheetCell()
 	if (!mpImpl->getState().mbInSheetCell) return;
 
 	mpImpl->popState();
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("table:table-cell"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("table:table-cell"));
 }
 
 void OdsGenerator::defineChartStyle(const librevenge::RVNGPropertyList &propList)
@@ -1162,7 +1202,7 @@ void OdsGenerator::openHeader(const librevenge::RVNGPropertyList &propList)
 	if (!mpImpl->inHeaderFooter())
 		return;
 
-	libodfgen::DocumentElementVector *pHeaderFooterContentElements = new libodfgen::DocumentElementVector;
+	auto pHeaderFooterContentElements = std::make_shared<libodfgen::DocumentElementVector>();
 	if (propList["librevenge:occurrence"] && (propList["librevenge:occurrence"]->getStr() == "even" ||
 	                                          propList["librevenge:occurrence"]->getStr() == "left"))
 		mpImpl->mpCurrentPageSpan->setHeaderLeftContent(pHeaderFooterContentElements);
@@ -1206,7 +1246,7 @@ void OdsGenerator::openFooter(const librevenge::RVNGPropertyList &propList)
 	if (!mpImpl->inHeaderFooter())
 		return;
 
-	libodfgen::DocumentElementVector *pHeaderFooterContentElements = new libodfgen::DocumentElementVector;
+	auto pHeaderFooterContentElements = std::make_shared<libodfgen::DocumentElementVector>();
 	if (propList["librevenge:occurrence"] && (propList["librevenge:occurrence"]->getStr() == "even" ||
 	                                          propList["librevenge:occurrence"]->getStr() == "left"))
 		mpImpl->mpCurrentPageSpan->setFooterLeftContent(pHeaderFooterContentElements);
@@ -1466,7 +1506,8 @@ void OdsGenerator::openComment(const librevenge::RVNGPropertyList &propList)
 
 	mpImpl->getState().mbInComment=true;
 	mpImpl->pushListState();
-	mpImpl->getCurrentStorage()->push_back(new TagOpenElement("office:annotation"));
+
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("office:annotation"));
 }
 
 void OdsGenerator::closeComment()
@@ -1480,7 +1521,7 @@ void OdsGenerator::closeComment()
 
 	if (mpImpl->mAuxiliarOdcState || !state.mbInComment) return;
 	mpImpl->popListState();
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("office:annotation"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("office:annotation"));
 }
 
 void OdsGenerator::openTable(const librevenge::RVNGPropertyList &propList)
@@ -1634,12 +1675,14 @@ void OdsGenerator::openFrame(const librevenge::RVNGPropertyList &propList)
 
 	if (!state.mbInSheetRow && !state.mbInSheetShapes)
 	{
-		mpImpl->getCurrentStorage()->push_back(new TagOpenElement("table:shapes"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("table:shapes"));
 		mpImpl->getState().mbInSheetShapes=prevState.mbInSheetShapes=true;
 	}
 
 	librevenge::RVNGPropertyList pList(propList);
-	if (!propList["text:anchor-type"])
+	if (!state.mbInSheetCell && propList["table:end-cell-address"])
+		pList.remove("table:end-cell-address");
+	if (!propList["text:anchor-type"] && !pList["table:end-cell-address"])
 		pList.insert("text:anchor-type","paragraph");
 	mpImpl->openFrame(pList);
 }
@@ -1672,6 +1715,22 @@ void OdsGenerator::insertBinaryObject(const librevenge::RVNGPropertyList &propLi
 	mpImpl->insertBinaryObject(propList);
 }
 
+void OdsGenerator::insertEquation(const librevenge::RVNGPropertyList &propList)
+{
+	// Embedded objects without a frame simply don't make sense for us
+	if (!mpImpl->getState().mbFirstInFrame)
+	{
+		ODFGEN_DEBUG_MSG(("OdsGenerator::insertEquation: called outsided a frame\n"));
+		return;
+	}
+	mpImpl->getState().mbFirstInFrame=false;
+	if (mpImpl->mAuxiliarOdtState)
+		return mpImpl->mAuxiliarOdtState->get().insertEquation(propList);
+	if (!mpImpl->checkOutsideOdc("insertEquation"))
+		return;
+	mpImpl->insertEquation(propList);
+}
+
 void OdsGenerator::openTextBox(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->open(OdsGeneratorPrivate::C_TextBox);
@@ -1692,7 +1751,7 @@ void OdsGenerator::openTextBox(const librevenge::RVNGPropertyList &propList)
 	if (!mpImpl->checkOutsideOdc("openTextBox"))
 		return;
 
-	TagOpenElement *textBoxOpenElement = new TagOpenElement("draw:text-box");
+	auto textBoxOpenElement = std::make_shared<TagOpenElement>("draw:text-box");
 	if (propList["librevenge:next-frame-name"])
 	{
 		librevenge::RVNGString frameName;
@@ -1716,10 +1775,10 @@ void OdsGenerator::closeTextBox()
 		return mpImpl->mAuxiliarOdtState->get().closeTextBox();
 	if (mpImpl->mAuxiliarOdcState || !state.mbInTextBox)
 		return;
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:text-box"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:text-box"));
 }
 
-void OdsGenerator::startDocument(const librevenge::RVNGPropertyList &)
+void OdsGenerator::startDocument(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->getState().mbStarted)
 	{
@@ -1728,6 +1787,7 @@ void OdsGenerator::startDocument(const librevenge::RVNGPropertyList &)
 	}
 	mpImpl->getState().mbStarted=true;
 	mpImpl->open(OdsGeneratorPrivate::C_Document);
+	mpImpl->appendBodySettings(propList);
 }
 
 void OdsGenerator::endDocument()
@@ -1755,7 +1815,7 @@ void OdsGenerator::endDocument()
 }
 
 
-void OdsGenerator::openGroup(const ::librevenge::RVNGPropertyList &propList)
+void OdsGenerator::openGroup(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->open(OdsGeneratorPrivate::C_Group);
 	if (mpImpl->mAuxiliarOdtState)
@@ -1780,14 +1840,14 @@ void OdsGenerator::closeGroup()
 	mpImpl->closeGroup();
 }
 
-void OdsGenerator::defineGraphicStyle(const ::librevenge::RVNGPropertyList &propList)
+void OdsGenerator::defineGraphicStyle(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->mAuxiliarOdtState)
 		return mpImpl->mAuxiliarOdtState->get().defineGraphicStyle(propList);
 	mpImpl->defineGraphicStyle(propList);
 }
 
-void OdsGenerator::drawRectangle(const ::librevenge::RVNGPropertyList &propList)
+void OdsGenerator::drawRectangle(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->mAuxiliarOdtState)
 		return mpImpl->mAuxiliarOdtState->get().drawRectangle(propList);
@@ -1796,7 +1856,7 @@ void OdsGenerator::drawRectangle(const ::librevenge::RVNGPropertyList &propList)
 	mpImpl->drawRectangle(propList);
 }
 
-void OdsGenerator::drawEllipse(const ::librevenge::RVNGPropertyList &propList)
+void OdsGenerator::drawEllipse(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->mAuxiliarOdtState)
 		return mpImpl->mAuxiliarOdtState->get().drawEllipse(propList);
@@ -1806,7 +1866,7 @@ void OdsGenerator::drawEllipse(const ::librevenge::RVNGPropertyList &propList)
 }
 
 
-void OdsGenerator::drawPolygon(const ::librevenge::RVNGPropertyList &propList)
+void OdsGenerator::drawPolygon(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->mAuxiliarOdtState)
 		return mpImpl->mAuxiliarOdtState->get().drawPolygon(propList);
@@ -1816,7 +1876,7 @@ void OdsGenerator::drawPolygon(const ::librevenge::RVNGPropertyList &propList)
 }
 
 
-void OdsGenerator::drawPolyline(const ::librevenge::RVNGPropertyList &propList)
+void OdsGenerator::drawPolyline(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->mAuxiliarOdtState)
 		return mpImpl->mAuxiliarOdtState->get().drawPolyline(propList);
@@ -1826,7 +1886,7 @@ void OdsGenerator::drawPolyline(const ::librevenge::RVNGPropertyList &propList)
 }
 
 
-void OdsGenerator::drawPath(const ::librevenge::RVNGPropertyList &propList)
+void OdsGenerator::drawPath(const librevenge::RVNGPropertyList &propList)
 {
 	if (mpImpl->mAuxiliarOdtState)
 		return mpImpl->mAuxiliarOdtState->get().drawPath(propList);
@@ -1835,7 +1895,7 @@ void OdsGenerator::drawPath(const ::librevenge::RVNGPropertyList &propList)
 	mpImpl->drawPath(propList);
 }
 
-void OdsGenerator::drawConnector(const ::librevenge::RVNGPropertyList &propList)
+void OdsGenerator::drawConnector(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawConnector(propList);
 }
