@@ -24,12 +24,13 @@
  * Corel Corporation or Corel Corporation Limited."
  */
 
+#include <libodfgen/libodfgen.hxx>
+
 #include <map>
 #include <stack>
 #include <string>
 
 #include <librevenge/librevenge.h>
-#include <libodfgen/libodfgen.hxx>
 
 #include "DocumentElement.hxx"
 #include "FilterInternal.hxx"
@@ -48,9 +49,9 @@ class OdtGeneratorPrivate : public OdfGenerator
 {
 public:
 	OdtGeneratorPrivate();
-	~OdtGeneratorPrivate();
+	~OdtGeneratorPrivate() override;
 
-	bool writeTargetDocument(OdfDocumentHandler *pHandler, OdfStreamType streamType);
+	bool writeTargetDocument(OdfDocumentHandler *pHandler, OdfStreamType streamType) override;
 	void _writeStyles(OdfDocumentHandler *pHandler);
 	void _writeAutomaticStyles(OdfDocumentHandler *pHandler, OdfStreamType streamType);
 
@@ -122,7 +123,7 @@ private:
 OdtGeneratorPrivate::OdtGeneratorPrivate() :
 	mStateStack(),
 	mSectionManager(),
-	mpCurrentPageSpan(0)
+	mpCurrentPageSpan(nullptr)
 {
 	initPageManager();
 	pushState();
@@ -146,6 +147,7 @@ void OdtGeneratorPrivate::_writeAutomaticStyles(OdfDocumentHandler *pHandler, Od
 		mParagraphManager.write(pHandler, Style::Z_StyleAutomatic);
 		mListManager.write(pHandler, Style::Z_StyleAutomatic);
 		mGraphicManager.write(pHandler, Style::Z_StyleAutomatic);
+		mNumberingManager.write(pHandler, Style::Z_StyleAutomatic);
 		mTableManager.write(pHandler, Style::Z_StyleAutomatic);
 	}
 	if ((streamType == ODF_FLAT_XML) || (streamType == ODF_CONTENT_XML))
@@ -156,6 +158,7 @@ void OdtGeneratorPrivate::_writeAutomaticStyles(OdfDocumentHandler *pHandler, Od
 		mParagraphManager.write(pHandler, Style::Z_ContentAutomatic);
 		mListManager.write(pHandler, Style::Z_ContentAutomatic);
 		mGraphicManager.write(pHandler, Style::Z_ContentAutomatic);
+		mNumberingManager.write(pHandler, Style::Z_ContentAutomatic);
 		mTableManager.write(pHandler, Style::Z_ContentAutomatic);
 	}
 
@@ -228,7 +231,7 @@ void OdtGeneratorPrivate::_writeStyles(OdfDocumentHandler *pHandler)
 	standardStyleOpenElement.write(pHandler);
 	pHandler->endElement("style:style");
 
-	static char const *(s_paraStyle[4*10]) =
+	static char const *s_paraStyle[4*10] =
 	{
 		"Text_Body", "Text Body", "Standard", "text",
 		"Table_Contents", "Table Contents", "Text_Body", "extra",
@@ -253,7 +256,7 @@ void OdtGeneratorPrivate::_writeStyles(OdfDocumentHandler *pHandler)
 		pHandler->endElement("style:style");
 	}
 
-	static char const *(s_textStyle[2*4])=
+	static char const *s_textStyle[2*4] =
 	{
 		"Footnote_Symbol", "Footnote Symbol", "Endnote_Symbol", "Endnote Symbol",
 		"Footnote_anchor", "Footnote anchor", "Endnote_anchor", "Endnote anchor"
@@ -284,7 +287,7 @@ void OdtGeneratorPrivate::_writeStyles(OdfDocumentHandler *pHandler)
 	lineOpenElement.write(pHandler);
 	pHandler->endElement("text:linenumbering-configuration");
 
-	static char const *(s_noteConfig[4*2])=
+	static char const *s_noteConfig[4*2] =
 	{
 		"footnote", "Footnote_Symbol", "Footnote_anchor", "1",
 		"endnote", "Endnote_Symbol", "Endnote_anchor", "i"
@@ -307,6 +310,7 @@ void OdtGeneratorPrivate::_writeStyles(OdfDocumentHandler *pHandler)
 		noteOpenElement.write(pHandler);
 		pHandler->endElement("text:notes-configuration");
 	}
+	mFillManager.write(pHandler);
 	mGraphicManager.write(pHandler, Style::Z_Style);
 	pHandler->endElement("office:styles");
 }
@@ -431,7 +435,7 @@ bool OdtGeneratorPrivate::writeTargetDocument(OdfDocumentHandler *pHandler, OdfS
 		// writing out the document
 		TagOpenElement("office:body").write(pHandler);
 		TagOpenElement("office:text").write(pHandler);
-		sendStorage(&mBodyStorage, pHandler);
+		sendStorage(mpBodyStorage.get(), pHandler);
 		ODFGEN_DEBUG_MSG(("OdtGenerator: Document Body: Finished writing all doc els..\n"));
 
 		pHandler->endElement("office:text");
@@ -480,9 +484,9 @@ void OdtGenerator::setDocumentMetaData(const librevenge::RVNGPropertyList &propL
 	mpImpl->setDocumentMetaData(propList);
 }
 
-void OdtGenerator::defineEmbeddedFont(const librevenge::RVNGPropertyList &/*propList*/)
+void OdtGenerator::defineEmbeddedFont(const librevenge::RVNGPropertyList &propList)
 {
-	// TODO: implement me
+	mpImpl->defineEmbeddedFont(propList);
 }
 
 void OdtGenerator::openPageSpan(const librevenge::RVNGPropertyList &propList)
@@ -503,7 +507,7 @@ void OdtGenerator::openHeader(const librevenge::RVNGPropertyList &propList)
 	if (!mpImpl->inHeaderFooter())
 		return;
 
-	libodfgen::DocumentElementVector *pHeaderFooterContentElements = new libodfgen::DocumentElementVector;
+	auto pHeaderFooterContentElements = std::make_shared<libodfgen::DocumentElementVector>();
 	if (propList["librevenge:occurrence"] && (propList["librevenge:occurrence"]->getStr() == "even" ||
 	                                          propList["librevenge:occurrence"]->getStr() == "left"))
 		mpImpl->mpCurrentPageSpan->setHeaderLeftContent(pHeaderFooterContentElements);
@@ -539,7 +543,7 @@ void OdtGenerator::openFooter(const librevenge::RVNGPropertyList &propList)
 	if (!mpImpl->inHeaderFooter())
 		return;
 
-	libodfgen::DocumentElementVector *pHeaderFooterContentElements = new libodfgen::DocumentElementVector;
+	auto pHeaderFooterContentElements = std::make_shared<libodfgen::DocumentElementVector>();;
 	if (propList["librevenge:occurrence"] && (propList["librevenge:occurrence"]->getStr() == "even" ||
 	                                          propList["librevenge:occurrence"]->getStr() == "left"))
 		mpImpl->mpCurrentPageSpan->setFooterLeftContent(pHeaderFooterContentElements);
@@ -581,7 +585,7 @@ void OdtGenerator::openSection(const librevenge::RVNGPropertyList &propList)
 	{
 		librevenge::RVNGString sSectionName=
 		    mpImpl->mSectionManager.add(propList, mpImpl->useStyleAutomaticZone() ? Style::Z_StyleAutomatic : Style::Z_Unknown);
-		TagOpenElement *pSectionOpenElement = new TagOpenElement("text:section");
+		auto pSectionOpenElement = std::make_shared<TagOpenElement>("text:section");
 		pSectionOpenElement->addAttribute("text:style-name", sSectionName);
 		pSectionOpenElement->addAttribute("text:name", sSectionName);
 		mpImpl->getCurrentStorage()->push_back(pSectionOpenElement);
@@ -593,7 +597,7 @@ void OdtGenerator::openSection(const librevenge::RVNGPropertyList &propList)
 void OdtGenerator::closeSection()
 {
 	if (!mpImpl->getState().mbInFakeSection)
-		mpImpl->getCurrentStorage()->push_back(new TagCloseElement("text:section"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("text:section"));
 	else
 		mpImpl->getState().mbInFakeSection = false;
 }
@@ -604,7 +608,7 @@ void OdtGenerator::openParagraph(const librevenge::RVNGPropertyList &propList)
 	// from "Table Contents"
 
 	librevenge::RVNGPropertyList finalPropList(propList);
-	if (mpImpl->getState().mbFirstParagraphInPageSpan && mpImpl->getCurrentStorage() == &mpImpl->getBodyStorage())
+	if (mpImpl->getState().mbFirstParagraphInPageSpan && mpImpl->getCurrentStorage() == mpImpl->getBodyStorage())
 	{
 		if (!mpImpl->mpCurrentPageSpan)
 		{
@@ -681,10 +685,21 @@ void OdtGenerator::closeUnorderedListLevel()
 
 void OdtGenerator::openListElement(const librevenge::RVNGPropertyList &propList)
 {
-	mpImpl->openListElement(propList);
-
-	if (mpImpl->getCurrentStorage() == &mpImpl->getBodyStorage())
-		mpImpl->getState().mbFirstParagraphInPageSpan = false;
+	librevenge::RVNGPropertyList finalPropList(propList);
+	if (mpImpl->getState().mbFirstParagraphInPageSpan && mpImpl->getCurrentStorage() == mpImpl->getBodyStorage())
+	{
+		if (!mpImpl->mpCurrentPageSpan)
+		{
+			ODFGEN_DEBUG_MSG(("OdtGenerator::openParagraph: can not find the current page\n"));
+		}
+		else
+		{
+			finalPropList.insert("style:master-page-name", mpImpl->mpCurrentPageSpan->getMasterName());
+			mpImpl->getState().mbFirstElement = false;
+			mpImpl->getState().mbFirstParagraphInPageSpan = false;
+		}
+	}
+	mpImpl->openListElement(finalPropList);
 }
 
 void OdtGenerator::closeListElement()
@@ -695,7 +710,7 @@ void OdtGenerator::closeListElement()
 void OdtGenerator::openFootnote(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->pushListState();
-	TagOpenElement *pOpenFootNote = new TagOpenElement("text:note");
+	auto pOpenFootNote = std::make_shared<TagOpenElement>("text:note");
 	pOpenFootNote->addAttribute("text:note-class", "footnote");
 	if (propList["librevenge:number"])
 	{
@@ -705,7 +720,7 @@ void OdtGenerator::openFootnote(const librevenge::RVNGPropertyList &propList)
 	}
 	mpImpl->getCurrentStorage()->push_back(pOpenFootNote);
 
-	TagOpenElement *pOpenFootCitation = new TagOpenElement("text:note-citation");
+	auto pOpenFootCitation = std::make_shared<TagOpenElement>("text:note-citation");
 	if (propList["text:label"])
 	{
 		librevenge::RVNGString tmpString;
@@ -715,12 +730,12 @@ void OdtGenerator::openFootnote(const librevenge::RVNGPropertyList &propList)
 	mpImpl->getCurrentStorage()->push_back(pOpenFootCitation);
 
 	if (propList["text:label"])
-		mpImpl->getCurrentStorage()->push_back(new CharDataElement(propList["text:label"]->getStr().cstr()));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<CharDataElement>(propList["text:label"]->getStr().cstr()));
 	else if (propList["librevenge:number"])
-		mpImpl->getCurrentStorage()->push_back(new CharDataElement(propList["librevenge:number"]->getStr().cstr()));
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("text:note-citation"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<CharDataElement>(propList["librevenge:number"]->getStr().cstr()));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("text:note-citation"));
 
-	mpImpl->getCurrentStorage()->push_back(new TagOpenElement("text:note-body"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("text:note-body"));
 
 	mpImpl->getState().mbInNote = true;
 }
@@ -730,14 +745,14 @@ void OdtGenerator::closeFootnote()
 	mpImpl->getState().mbInNote = false;
 	mpImpl->popListState();
 
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("text:note-body"));
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("text:note"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("text:note-body"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("text:note"));
 }
 
 void OdtGenerator::openEndnote(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->pushListState();
-	TagOpenElement *pOpenEndNote = new TagOpenElement("text:note");
+	auto pOpenEndNote = std::make_shared<TagOpenElement>("text:note");
 	pOpenEndNote->addAttribute("text:note-class", "endnote");
 	if (propList["librevenge:number"])
 	{
@@ -747,7 +762,7 @@ void OdtGenerator::openEndnote(const librevenge::RVNGPropertyList &propList)
 	}
 	mpImpl->getCurrentStorage()->push_back(pOpenEndNote);
 
-	TagOpenElement *pOpenEndCitation = new TagOpenElement("text:note-citation");
+	auto pOpenEndCitation = std::make_shared<TagOpenElement>("text:note-citation");
 	if (propList["text:label"])
 	{
 		librevenge::RVNGString tmpString;
@@ -757,12 +772,12 @@ void OdtGenerator::openEndnote(const librevenge::RVNGPropertyList &propList)
 	mpImpl->getCurrentStorage()->push_back(pOpenEndCitation);
 
 	if (propList["text:label"])
-		mpImpl->getCurrentStorage()->push_back(new CharDataElement(propList["text:label"]->getStr().cstr()));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<CharDataElement>(propList["text:label"]->getStr().cstr()));
 	else if (propList["librevenge:number"])
-		mpImpl->getCurrentStorage()->push_back(new CharDataElement(propList["librevenge:number"]->getStr().cstr()));
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("text:note-citation"));
+		mpImpl->getCurrentStorage()->push_back(std::make_shared<CharDataElement>(propList["librevenge:number"]->getStr().cstr()));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("text:note-citation"));
 
-	mpImpl->getCurrentStorage()->push_back(new TagOpenElement("text:note-body"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("text:note-body"));
 
 	mpImpl->getState().mbInNote = true;
 }
@@ -771,14 +786,14 @@ void OdtGenerator::closeEndnote()
 {
 	mpImpl->getState().mbInNote = false;
 	mpImpl->popListState();
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("text:note-body"));
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("text:note"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("text:note-body"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("text:note"));
 }
 
 void OdtGenerator::openComment(const librevenge::RVNGPropertyList &)
 {
 	mpImpl->pushListState();
-	mpImpl->getCurrentStorage()->push_back(new TagOpenElement("office:annotation"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagOpenElement>("office:annotation"));
 
 	mpImpl->getState().mbInNote = true;
 }
@@ -787,7 +802,7 @@ void OdtGenerator::closeComment()
 {
 	mpImpl->getState().mbInNote = false;
 	mpImpl->popListState();
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("office:annotation"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("office:annotation"));
 }
 
 void OdtGenerator::openTable(const librevenge::RVNGPropertyList &propList)
@@ -796,7 +811,7 @@ void OdtGenerator::openTable(const librevenge::RVNGPropertyList &propList)
 		return;
 
 	librevenge::RVNGPropertyList pList(propList);
-	if (mpImpl->getState().mbFirstElement && mpImpl->getCurrentStorage() == &mpImpl->getBodyStorage())
+	if (mpImpl->getState().mbFirstElement && mpImpl->getCurrentStorage() == mpImpl->getBodyStorage())
 	{
 		if (!mpImpl->mpCurrentPageSpan)
 		{
@@ -904,7 +919,7 @@ void OdtGenerator::insertBinaryObject(const librevenge::RVNGPropertyList &propLi
 	mpImpl->insertBinaryObject(propList);
 }
 
-void OdtGenerator::openGroup(const ::librevenge::RVNGPropertyList &propList)
+void OdtGenerator::openGroup(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->openGroup(propList);
 }
@@ -914,35 +929,35 @@ void OdtGenerator::closeGroup()
 	mpImpl->closeGroup();
 }
 
-void OdtGenerator::defineGraphicStyle(const ::librevenge::RVNGPropertyList &propList)
+void OdtGenerator::defineGraphicStyle(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->defineGraphicStyle(propList);
 }
 
-void OdtGenerator::drawRectangle(const ::librevenge::RVNGPropertyList &propList)
+void OdtGenerator::drawRectangle(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawRectangle(propList);
 }
 
-void OdtGenerator::drawEllipse(const ::librevenge::RVNGPropertyList &propList)
+void OdtGenerator::drawEllipse(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawEllipse(propList);
 }
 
 
-void OdtGenerator::drawPolygon(const ::librevenge::RVNGPropertyList &propList)
+void OdtGenerator::drawPolygon(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawPolySomething(propList, true);
 }
 
 
-void OdtGenerator::drawPolyline(const ::librevenge::RVNGPropertyList &propList)
+void OdtGenerator::drawPolyline(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawPolySomething(propList, false);
 }
 
 
-void OdtGenerator::drawPath(const ::librevenge::RVNGPropertyList &propList)
+void OdtGenerator::drawPath(const librevenge::RVNGPropertyList &propList)
 {
 	/** CHECKME: actually, the drawing is often bad, ie. some
 	 unexpected scaling be applied, can we automatically replace the
@@ -950,7 +965,7 @@ void OdtGenerator::drawPath(const ::librevenge::RVNGPropertyList &propList)
 	mpImpl->drawPath(propList);
 }
 
-void OdtGenerator::drawConnector(const ::librevenge::RVNGPropertyList &propList)
+void OdtGenerator::drawConnector(const librevenge::RVNGPropertyList &propList)
 {
 	mpImpl->drawConnector(propList);
 }
@@ -961,7 +976,7 @@ void OdtGenerator::openTextBox(const librevenge::RVNGPropertyList &propList)
 		return;
 	mpImpl->pushListState();
 	mpImpl->pushState();
-	TagOpenElement *textBoxOpenElement = new TagOpenElement("draw:text-box");
+	auto textBoxOpenElement = std::make_shared<TagOpenElement>("draw:text-box");
 	if (propList["librevenge:next-frame-name"])
 	{
 		librevenge::RVNGString frameName;
@@ -981,15 +996,18 @@ void OdtGenerator::closeTextBox()
 	mpImpl->popListState();
 	mpImpl->popState();
 
-	mpImpl->getCurrentStorage()->push_back(new TagCloseElement("draw:text-box"));
+	mpImpl->getCurrentStorage()->push_back(std::make_shared<TagCloseElement>("draw:text-box"));
 }
 
 void OdtGenerator::defineSectionStyle(librevenge::RVNGPropertyList const &)
 {
 }
 
-void OdtGenerator::insertEquation(librevenge::RVNGPropertyList const &)
+void OdtGenerator::insertEquation(librevenge::RVNGPropertyList const &propList)
 {
+	if (!mpImpl->getState().mbInFrame) // Embedded objects without a frame simply don't make sense for us
+		return;
+	mpImpl->insertEquation(propList);
 }
 
 void OdtGenerator::endDocument()
@@ -998,8 +1016,9 @@ void OdtGenerator::endDocument()
 	mpImpl->writeTargetDocuments();
 }
 
-void OdtGenerator::startDocument(const librevenge::RVNGPropertyList &)
+void OdtGenerator::startDocument(const librevenge::RVNGPropertyList &propList)
 {
+	mpImpl->appendBodySettings(propList);
 }
 
 void OdtGenerator::closePageSpan()
